const lo = require('lodash')
const hlpPolicy = require('../../helpers_policy')

const TaskTypeSettingsValue = 'PluginTaskType'
const taskPolicyConverters = {
    //OnAccess: require('./types/OnAccess.js')
}

const taskConverters = {
    //"remove-file": require('./types/remove-file.js'),
    "quarantine-file": require('./types/quarantine-file.js'),
    //"restore-file": require('./types/restore-file.js'),
    //"ioc-scan": require('./types/ioc-scan.js'),
    //"terminate-process": require('./types/terminate-process.js'),
    //"execute-command": require('./types/execute-command.js')
}

function LocalTaskSettings () {
    let curHostId = ''
    let curTaskId = ''
    let currentTaskType = ''
    let activePolicyId = -1
    let taskAdditionalParams = {}
    let policiesCache = {}
    let rbacCache = {}

    async function getTaskDataCache(kscApi, taskId, hostId) {
        let result = await hlpPolicy.GetTaskData(kscApi, taskId, hostId);
        if ( result && result.TASK_ADDITIONAL_PARAMS !==undefined ) {
            taskAdditionalParams = lo.clone(result.TASK_ADDITIONAL_PARAMS)
            currentTaskType = result.TASK_NAME
        }
        else{
            curTaskId = ''
            currentTaskType = ''
            taskAdditionalParams = {}
        }
    }

    async function GetRbacCache (api, section) {
        /*
        try{
            let ReadonlyNodes = {}
            await api.HstAccessControl.GetSettingsReadonlyNodes({
                lGroupId: curHostId,
                szwProduct: consts.wstrProduct,
                szwProduct: consts.wstrVersion,
                szwSectionName: settSections[section],
                pSettingsSection:ReadonlyNodes
            })
            return ReadonlyNodes;
        }
        catch (error) {
            console.error(error)
        }
         */
        return {};
    }

    /*
     * Called via api.js in getFormModel
     */
    async function getTaskData(kscApi, taskId, hostId, taskType, form) {
        if(curHostId!==hostId || curTaskId!= taskId) {
            curHostId = hostId
            curTaskId = taskId
            taskAdditionalParams = {}
            policiesCache = {}
        }

        // request from Root task page form with TaskType
        let converterPolicy = taskPolicyConverters[currentTaskType]

        await getTaskDataCache(kscApi, taskId, hostId);

        if(converterPolicy)
        {
            activePolicyId = await hlpPolicy.GetHostActivePolicyId(kscApi, hostId);
            if(activePolicyId===-1){
                hlpPolicy.UpdatePoliciesCacheAsDefault(taskPolicyConverters[currentTaskType].defPolicySections, policiesCache);
            } else {
                await hlpPolicy.UpdatePoliciesCache(kscApi, activePolicyId, taskPolicyConverters[currentTaskType].defPolicySections, policiesCache);
            }
        }

        // get data from cache
        let computedData = {};

        if(converterPolicy) {
            if(converterPolicy.toViewEx!==undefined){
                computedData = await converterPolicy.toViewEx(kscApi, taskAdditionalParams, policiesCache, form);
            } else{
                computedData = converterPolicy.toView(taskAdditionalParams, policiesCache, form);
            }
            computedData[TaskTypeSettingsValue] = currentTaskType;
        }

        let converter = taskConverters[currentTaskType]
        if(converter) {
            if(converter.toViewEx!==undefined){
                computedData = await converter.toViewEx(kscApi, taskAdditionalParams, form);
            } else{
                computedData = converter.toView(taskAdditionalParams, form);
            }
            computedData[TaskTypeSettingsValue] = currentTaskType;
        }

        return computedData;
    }

    function getTaskAdditionalParams(TaskType) {
        console.log('ksws_windows:LocalTask:getTaskAdditionalParams ' + TaskType + ', Params:' + JSON.stringify(taskAdditionalParams))
        return taskAdditionalParams;
    }

    async function updateTaskData ({kscApi, modelData, formType}) {
        let converter = taskPolicyConverters[currentTaskType]
        if (converter) {
            converter.toRaw(taskAdditionalParams, modelData, formType)
        }
    }

    return {
        getTaskData,
        getTaskAdditionalParams,
        updateTaskData
    }
}

module.exports = LocalTaskSettings
