const { helpers } = require('@kl/openapi-client')

const lo = require('lodash')
const fs = require('fs')
const path = require('path')

const consts = require('./consts')
const hlp = require('./helpers')

function RemoveMandatories(item) {
    Object.keys(item).forEach(property => {
        if(item[property].KLPRSS_Mnd!==undefined){
            item[property].KLPRSS_Mnd = false
        }
        if(item[property].KLPRSS_Val!== undefined &&
                typeof item[property].KLPRSS_Val[0]==='object' &&
                    !Array.isArray(item[property].KLPRSS_Val[0]) ){
            RemoveMandatories(item[property].KLPRSS_Val[0]);
        }
    })
}

function DefaultPolicyObject(policySection, sectionMapper){
    try {
        const policyShortName = sectionMapper!==undefined ? sectionMapper[policySection] : policySection;
        const parsedFileName = path.parse(policyShortName + '.json');
        const filePath = path.join(__dirname, 'converters\\policy\\defaults', parsedFileName.base);
        const DefaultsRaw = fs.readFileSync(filePath, 'utf-8');
        return hlp.deserialize({PxgRetVal:JSON.parse(DefaultsRaw)}).KLPRSS_Val;
    } catch (error) {
        console.log('ksws_windows:> Failed to read default section ' + policySection + '.json')
    }
    return {}
}

async function GetHostActivePolicyId(kscApi, hostId) {
    let policy = await kscApi.utils.getHostActivePolicy({
        hostName:hostId,
        productId: consts.wstrProduct,
        productVersion: consts.wstrVersion,
    });
    return (policy!==undefined && policy.KLPOL_ID!==undefined ) ? policy.KLPOL_ID : -1
}

function UpdatePoliciesCacheAsDefault(arrSections, policyCache, sectionMapper) {
    arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]
    for (let section of arrSections) {
        if( section==='' )
            continue;
        const rowPolicy = DefaultPolicyObject(section, sectionMapper);
        RemoveMandatories(rowPolicy);
        policyCache[section] = rowPolicy;
    }
}

async function UpdatePoliciesCache(kscApi, policyId, arrSections, policyCache, sectionMapper) {
    if (policyId === -1)
        return

    arrSections = Array.isArray(arrSections) ? arrSections : [arrSections];
    if(sectionMapper !== undefined)
    {
        arrSections = arrSections.map(function (item) {
            return sectionMapper[item]
        })
    }

    let { result: identifier } = await kscApi.Policy.GetPolicyContents({nPolicy:policyId})
    for (let section of arrSections) {
        if( consts.policySsSections[section]===undefined )
            continue

        let sectionData = await kscApi.SsContents.Ss_Read({
            wstrID: identifier,
            wstrProduct: consts.wstrProduct,
            wstrVersion: consts.wstrVersion,
            wstrSection: consts.policySsSections[section],
            skipDeserialization: true});

        if(sectionData.result!==undefined){
            policyCache[section] = hlp.deserialize(lo.get(sectionData, 'result'));
        } else {
            policyCache[section] = DefaultPolicyObject(section);
        }
    }
    kscApi.SsContents.Ss_Release({ wstrID: identifier })
}

async function GetTaskData(kscApi, taskId, hostId) {
    let result = {}
    const strTaskId = taskId.toString();
    try {
        let task = {}
        if (hostId){
            const strHostName = hostId.toString()
            let {result: strSrvObjId} = await kscApi.HostGroup.GetHostTasks({strHostName})
            task = await kscApi.HostTasks.GetTaskData({strTask:strTaskId, strSrvObjId})
            result = task.result
        }
        else {
            task = await kscApi.Tasks.GetTaskData({taskId:strTaskId, skipDeserialization:true})
            result = helpers.serializer.deserializeKscRequest({body: task.result}).result
        }
    } catch (e) {
        console.error(e);
    }
    return result
}

module.exports = {
    DefaultPolicyObject,
    GetHostActivePolicyId,
    UpdatePoliciesCache,
    UpdatePoliciesCacheAsDefault,
    GetTaskData
}